@@ -915,7 +915,7 @@ void cbmapclose(CBMAP *map){
 int cbmapput(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz, int over){
   CBMAPDATUM *datum, **entp, *old;
   char *dbuf;
-  int bidx, hash, kcmp;
+  int bidx, hash, kcmp, psiz;
   assert(map && kbuf && vbuf);
   if(ksiz < 0) ksiz = strlen(kbuf);
   if(vsiz < 0) vsiz = strlen(vbuf);
@@ -942,9 +942,10 @@ int cbmapput(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
         datum = datum->right;
       } else {
         if(!over) return FALSE;
+        psiz = CB_ALIGNPAD(ksiz);
         if(vsiz > datum->vsiz){
           old = datum;
-          CB_REALLOC(datum, sizeof(*datum) + ksiz + 1 + vsiz + 1);
+          CB_REALLOC(datum, sizeof(*datum) + ksiz + psiz + vsiz + 1);
           if(datum != old){
             if(map->first == old) map->first = datum;
             if(map->last == old) map->last = datum;
@@ -954,20 +955,21 @@ int cbmapput(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
             dbuf = (char *)datum + sizeof(*datum);
           }
         }
-        memcpy(dbuf + ksiz + 1, vbuf, vsiz);
-        dbuf[ksiz+1+vsiz] = '\0';
+        memcpy(dbuf + ksiz + psiz, vbuf, vsiz);
+        dbuf[ksiz+psiz+vsiz] = '\0';
         datum->vsiz = vsiz;
         return TRUE;
       }
     }
   }
-  CB_MALLOC(datum, sizeof(*datum) + ksiz + 1 + vsiz + 1);
+  psiz = CB_ALIGNPAD(ksiz);
+  CB_MALLOC(datum, sizeof(*datum) + ksiz + psiz + vsiz + 1);
   dbuf = (char *)datum + sizeof(*datum);
   memcpy(dbuf, kbuf, ksiz);
   dbuf[ksiz] = '\0';
   datum->ksiz = ksiz;
-  memcpy(dbuf + ksiz + 1, vbuf, vsiz);
-  dbuf[ksiz+1+vsiz] = '\0';
+  memcpy(dbuf + ksiz + psiz, vbuf, vsiz);
+  dbuf[ksiz+psiz+vsiz] = '\0';
   datum->vsiz = vsiz;
   datum->hash = hash;
   datum->left = NULL;
@@ -987,7 +989,7 @@ int cbmapput(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
 void cbmapputcat(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz){
   CBMAPDATUM *datum, **entp, *old;
   char *dbuf;
-  int bidx, hash, kcmp, asiz, unit;
+  int bidx, hash, kcmp, psiz, asiz, unit;
   assert(map && kbuf && vbuf);
   if(ksiz < 0) ksiz = strlen(kbuf);
   if(vsiz < 0) vsiz = strlen(vbuf);
@@ -1013,7 +1015,8 @@ void cbmapputcat(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int v
         entp = &(datum->right);
         datum = datum->right;
       } else {
-        asiz = sizeof(*datum) + ksiz + 1 + datum->vsiz + vsiz + 1;
+        psiz = CB_ALIGNPAD(ksiz);
+        asiz = sizeof(*datum) + ksiz + psiz + datum->vsiz + vsiz + 1;
         unit = asiz <= CB_MAPCSUNIT ? CB_MAPCSUNIT : CB_MAPCBUNIT;
         asiz = (asiz - 1) + unit - (asiz - 1) % unit;
         old = datum;
@@ -1026,14 +1029,15 @@ void cbmapputcat(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int v
           if(datum->next) datum->next->prev = datum;
           dbuf = (char *)datum + sizeof(*datum);
         }
-        memcpy(dbuf + ksiz + 1 + datum->vsiz, vbuf, vsiz);
-        dbuf[ksiz+1+datum->vsiz+vsiz] = '\0';
+        memcpy(dbuf + ksiz + psiz + datum->vsiz, vbuf, vsiz);
+        dbuf[ksiz+psiz+datum->vsiz+vsiz] = '\0';
         datum->vsiz += vsiz;
         return;
       }
     }
   }
-  asiz = sizeof(*datum) + ksiz + 1 + vsiz + 1;
+  psiz = CB_ALIGNPAD(ksiz);
+  asiz = sizeof(*datum) + ksiz + psiz + vsiz + 1;
   unit = asiz <= CB_MAPCSUNIT ? CB_MAPCSUNIT : CB_MAPCBUNIT;
   asiz = (asiz - 1) + unit - (asiz - 1) % unit;
   CB_MALLOC(datum, asiz);
@@ -1041,8 +1045,8 @@ void cbmapputcat(CBMAP *map, const char *kbuf, int ksiz, const char *vbuf, int v
   memcpy(dbuf, kbuf, ksiz);
   dbuf[ksiz] = '\0';
   datum->ksiz = ksiz;
-  memcpy(dbuf + ksiz + 1, vbuf, vsiz);
-  dbuf[ksiz+1+vsiz] = '\0';
+  memcpy(dbuf + ksiz + psiz, vbuf, vsiz);
+  dbuf[ksiz+psiz+vsiz] = '\0';
   datum->vsiz = vsiz;
   datum->hash = hash;
   datum->left = NULL;
@@ -1168,7 +1172,7 @@ const char *cbmapget(const CBMAP *map, const char *kbuf, int ksiz, int *sp){
         datum = datum->right;
       } else {
         if(sp) *sp = datum->vsiz;
-        return dbuf + datum->ksiz + 1;
+        return dbuf + datum->ksiz + CB_ALIGNPAD(datum->ksiz);
       }
     }
   }
@@ -1251,7 +1255,7 @@ const char *cbmapiterval(const char *kbuf, int *sp){
   assert(kbuf);
   datum = (CBMAPDATUM *)(kbuf - sizeof(*datum));
   if(sp) *sp = datum->vsiz;
-  return (char *)datum + sizeof(*datum) + datum->ksiz + 1;
+  return (char *)datum + sizeof(*datum) + datum->ksiz + CB_ALIGNPAD(datum->ksiz);;
 }
 
 
@@ -2360,6 +2364,9 @@ char *cbmimebreak(const char *ptr, int size, CBMAP *attrs, int *sp){
           }
           if(cbstrfwimatch(ep, "charset=")){
             ep += 8;
+            while(*ep > '\0' && *ep <= ' '){
+              ep++;
+            }
             if(ep[0] == '"') ep++;
             pv = ep;
             while(ep[0] != '\0' && ep[0] != ' ' && ep[0] != '"' && ep[0] != ';'){
@@ -2368,6 +2375,9 @@ char *cbmimebreak(const char *ptr, int size, CBMAP *attrs, int *sp){
             cbmapput(attrs, "CHARSET", -1, pv, ep - pv, TRUE);
           } else if(cbstrfwimatch(ep, "boundary=")){
             ep += 9;
+            while(*ep > '\0' && *ep <= ' '){
+              ep++;
+            }
             if(ep[0] == '"'){
               ep++;
               pv = ep;
@@ -2432,7 +2442,7 @@ CBLIST *cbmimeparts(const char *ptr, int size, const char *boundary){
   assert(ptr && boundary);
   if(size < 0) size = strlen(ptr);
   CB_LISTOPEN(list);
-  blen = strlen(boundary);
+  if((blen = strlen(boundary)) < 1) return list;
   pv = NULL;
   for(i = 0; i < size; i++){
     if(ptr[i] == '-' && ptr[i+1] == '-' && i + 2 + blen < size &&
@@ -2452,7 +2462,7 @@ CBLIST *cbmimeparts(const char *ptr, int size, const char *boundary){
       ep = ptr + i;
       if(ep > ptr && ep[-1] == '\n') ep--;
       if(ep > ptr && ep[-1] == '\r') ep--;
-      CB_LISTPUSH(list, pv, ep - pv);
+      if(ep > pv) CB_LISTPUSH(list, pv, ep - pv);
       pv = ptr + i + 2 + blen;
       if(*pv == '\r') pv++;
       if(*pv == '\n') pv++;
