@@ -40,6 +40,43 @@
 #define DP_NUMBUFSIZ   32                /* size of a buffer for a number */
 #define DP_IOBUFSIZ    8192              /* size of an I/O buffer */
 
+/* get the first hash value */
+#define DP_FIRSTHASH(DP_res, DP_kbuf, DP_ksiz) \
+  do { \
+    int _DP_i; \
+    if((DP_ksiz) == sizeof(int)){ \
+      memcpy(&(DP_res), (DP_kbuf), sizeof(int)); \
+    } else { \
+      (DP_res) = 751; \
+    } \
+    for(_DP_i = 0; _DP_i < (DP_ksiz); _DP_i++){ \
+      (DP_res) = (DP_res) * 31 + ((const unsigned char *)(DP_kbuf))[_DP_i]; \
+    } \
+    (DP_res) = ((DP_res) * 87767623) & 0x7FFFFFFF; \
+  } while(FALSE)
+
+/* get the second hash value */
+#define DP_SECONDHASH(DP_res, DP_kbuf, DP_ksiz) \
+  do { \
+    int _DP_i; \
+    (DP_res) = 19780211; \
+    for(_DP_i = (DP_ksiz) - 1; _DP_i >= 0; _DP_i--){ \
+    (DP_res) = (DP_res) * 37 + ((const unsigned char *)(DP_kbuf))[_DP_i]; \
+    } \
+    (DP_res) = ((DP_res) * 43321879) & 0x7FFFFFFF; \
+  } while(FALSE)
+
+/* get the third hash value */
+#define DP_THIRDHASH(DP_res, DP_kbuf, DP_ksiz) \
+  do { \
+    int _DP_i; \
+    (DP_res) = 774831917; \
+    for(_DP_i = (DP_ksiz) - 1; _DP_i >= 0; _DP_i--){ \
+      (DP_res) = (DP_res) * 29 + ((const unsigned char *)(DP_kbuf))[_DP_i]; \
+    } \
+    (DP_res) = ((DP_res) * 5157883) & 0x7FFFFFFF; \
+  } while(FALSE)
+
 enum {                                   /* enumeration for a record header */
   DP_RHIFLAGS,                           /* offset of flags */
   DP_RHIHASH,                            /* offset of value of the second hash function */
@@ -67,9 +104,6 @@ static int dpseekwritenum(int fd, int off, int num);
 static int dpread(int fd, void *buf, int size);
 static int dpseekread(int fd, int off, void *buf, int size);
 static int dpfcopy(int destfd, int destoff, int srcfd, int srcoff);
-static int dpfirsthash(const char *kbuf, int ksiz);
-static int dpsecondhash(const char *kbuf, int ksiz);
-static int dpthirdhash(const char *kbuf, int ksiz);
 static int dpgetprime(int num);
 static int dppadsize(DEPOT *depot, int ksiz, int vsiz);
 static int dprecsize(int *head);
@@ -320,7 +354,7 @@ int dpput(DEPOT *depot, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
   if(ksiz < 0) ksiz = strlen(kbuf);
   if(vsiz < 0) vsiz = strlen(vbuf);
   newoff = -1;
-  hash = dpsecondhash(kbuf, ksiz);
+  DP_SECONDHASH(hash, kbuf, ksiz);
   switch(dprecsearch(depot, kbuf, ksiz, hash, &bi, &off, &entoff, head, ebuf, &ee, TRUE)){
   case -1:
     depot->fatal = TRUE;
@@ -469,7 +503,7 @@ int dpout(DEPOT *depot, const char *kbuf, int ksiz){
     return FALSE;
   }
   if(ksiz < 0) ksiz = strlen(kbuf);
-  hash = dpsecondhash(kbuf, ksiz);
+  DP_SECONDHASH(hash, kbuf, ksiz);
   switch(dprecsearch(depot, kbuf, ksiz, hash, &bi, &off, &entoff, head, ebuf, &ee, FALSE)){
   case -1:
     depot->fatal = TRUE;
@@ -499,7 +533,7 @@ char *dpget(DEPOT *depot, const char *kbuf, int ksiz, int start, int max, int *s
     return NULL;
   }
   if(ksiz < 0) ksiz = strlen(kbuf);
-  hash = dpsecondhash(kbuf, ksiz);
+  DP_SECONDHASH(hash, kbuf, ksiz);
   switch(dprecsearch(depot, kbuf, ksiz, hash, &bi, &off, &entoff, head, ebuf, &ee, FALSE)){
   case -1:
     depot->fatal = TRUE;
@@ -555,7 +589,7 @@ int dpgetwb(DEPOT *depot, const char *kbuf, int ksiz, int start, int max, char *
     return -1;
   }
   if(ksiz < 0) ksiz = strlen(kbuf);
-  hash = dpsecondhash(kbuf, ksiz);
+  DP_SECONDHASH(hash, kbuf, ksiz);
   switch(dprecsearch(depot, kbuf, ksiz, hash, &bi, &off, &entoff, head, ebuf, &ee, FALSE)){
   case -1:
     depot->fatal = TRUE;
@@ -594,7 +628,7 @@ int dpvsiz(DEPOT *depot, const char *kbuf, int ksiz){
     return -1;
   }
   if(ksiz < 0) ksiz = strlen(kbuf);
-  hash = dpsecondhash(kbuf, ksiz);
+  DP_SECONDHASH(hash, kbuf, ksiz);
   switch(dprecsearch(depot, kbuf, ksiz, hash, &bi, &off, &entoff, head, ebuf, &ee, FALSE)){
   case -1:
     depot->fatal = TRUE;
@@ -1275,8 +1309,9 @@ char *dpsnaffle(const char *name, const char* kbuf, int ksiz, int *sp){
   err = FALSE;
   vbuf = NULL;
   vsiz = 0;
-  hash = dpsecondhash(kbuf, ksiz);
-  off = buckets[dpfirsthash(kbuf, ksiz)%bnum];
+  DP_SECONDHASH(hash, kbuf, ksiz);
+  DP_FIRSTHASH(thash, kbuf, ksiz);
+  off = buckets[thash%bnum];
   while(off != 0){
     if(!dpseekread(fd, off, head, DP_RHNUM * sizeof(int))){
       err = TRUE;
@@ -1345,17 +1380,21 @@ char *dpsnaffle(const char *name, const char* kbuf, int ksiz, int *sp){
 
 /* Hash function used inside Depot. */
 int dpinnerhash(const char *kbuf, int ksiz){
+  int res;
   assert(kbuf);
   if(ksiz < 0) ksiz = strlen(kbuf);
-  return dpfirsthash(kbuf, ksiz);
+  DP_FIRSTHASH(res, kbuf, ksiz);
+  return res;
 }
 
 
 /* Hash function which is independent from the hash functions used inside Depot. */
 int dpouterhash(const char *kbuf, int ksiz){
+  int res;
   assert(kbuf);
   if(ksiz < 0) ksiz = strlen(kbuf);
-  return dpthirdhash(kbuf, ksiz);
+  DP_THIRDHASH(res, kbuf, ksiz);
+  return res;
 }
 
 
@@ -1459,11 +1498,10 @@ int dpmemflush(DEPOT *depot){
 
 /* Get flags of a database. */
 int dpgetflags(DEPOT *depot){
+  int flags;
   assert(depot);
-  return (*(unsigned char *)(depot->map + DP_FLAGSOFF + 0) << 0) +
-    (*(unsigned char *)(depot->map + DP_FLAGSOFF + 1) << 8) +
-    (*(unsigned char *)(depot->map + DP_FLAGSOFF + 2) << 16) +
-    (*(unsigned char *)(depot->map + DP_FLAGSOFF + 3) << 24);
+  memcpy(&flags, depot->map + DP_FLAGSOFF, sizeof(int));
+  return flags;
 }
 
 
@@ -1474,10 +1512,7 @@ int dpsetflags(DEPOT *depot, int flags){
     dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
-  *(depot->map + DP_FLAGSOFF + 0) = (flags >> 0) & 0xff;
-  *(depot->map + DP_FLAGSOFF + 1) = (flags >> 8) & 0xff;
-  *(depot->map + DP_FLAGSOFF + 2) = (flags >> 16) & 0xff;
-  *(depot->map + DP_FLAGSOFF + 3) = (flags >> 24) & 0xff;
+  memcpy(depot->map + DP_FLAGSOFF, &flags, sizeof(int));
   return TRUE;
 }
 
@@ -1669,58 +1704,6 @@ static int dpfcopy(int destfd, int destoff, int srcfd, int srcoff){
 }
 
 
-/* Get the first hash value.
-   `kbuf' specifies the pointer to the region of a key.
-   `ksiz' specifies the size of the key.
-   The return value is 31 bit hash value of the key. */
-static int dpfirsthash(const char *kbuf, int ksiz){
-  unsigned int sum;
-  int i;
-  assert(kbuf && ksiz >= 0);
-  if(ksiz == sizeof(int)){
-    memcpy(&sum, kbuf, sizeof(int));
-  } else {
-    sum = 751;
-  }
-  for(i = 0; i < ksiz; i++){
-    sum = sum * 31 + ((const unsigned char *)kbuf)[i];
-  }
-  return (sum * 87767623) & 0x7FFFFFFF;
-}
-
-
-/* Get the second hash value.
-   `kbuf' specifies the pointer to the region of a key.
-   `ksiz' specifies the size of the key.
-   The return value is 31 bit hash value of the key. */
-static int dpsecondhash(const char *kbuf, int ksiz){
-  unsigned int sum;
-  int i;
-  assert(kbuf && ksiz >= 0);
-  sum = 19780211;
-  for(i = ksiz - 1; i >= 0; i--){
-    sum = sum * 37 + ((const unsigned char *)kbuf)[i];
-  }
-  return (sum * 43321879) & 0x7FFFFFFF;
-}
-
-
-/* Get the third hash value.
-   `kbuf' specifies the pointer to the region of a key.
-   `ksiz' specifies the size of the key.
-   The return value is 31 bit hash value of the key. */
-static int dpthirdhash(const char *kbuf, int ksiz){
-  unsigned int sum;
-  int i;
-  assert(kbuf && ksiz >= 0);
-  sum = 774831917;
-  for(i = ksiz - 1; i >= 0; i--){
-    sum = sum * 29 + ((const unsigned char *)kbuf)[i];
-  }
-  return (sum * 5157883) & 0x7FFFFFFF;
-}
-
-
 /* Get a natural prime number not less than a number.
    `num' specified a natural number.
    The return value is a prime number not less than the specified number. */
@@ -1935,7 +1918,8 @@ static int dprecsearch(DEPOT *depot, const char *kbuf, int ksiz, int hash, int *
   int off, entoff, thash, kcmp;
   char stkey[DP_STKBUFSIZ], *tkey;
   assert(depot && kbuf && ksiz >= 0 && hash >= 0 && bip && offp && entp && head && ebuf && eep);
-  *bip = dpfirsthash(kbuf, ksiz) % depot->bnum;
+  DP_FIRSTHASH(thash, kbuf, ksiz);
+  *bip = thash % depot->bnum;
   off = depot->buckets[*bip];
   *offp = -1;
   *entp = -1;
