@@ -133,6 +133,21 @@ public class Curia implements ADBM {
       if(crremove(name) == 0) throw new CuriaException(crecode());
     }
   }
+  /**
+   * Retrieve a record directly from a database directory.
+   * @param name the name of a database directory.
+   * @param key a byte array of a key.
+   * @throws CuriaException if an error occurs or no record corresponds.
+   * @note Although this method can be used even while the database directory is locked by
+   * another process, it is not assured that recent updated is reflected.
+   */
+  public static byte[] snaffle(String name, byte[] key) throws CuriaException {
+    synchronized(ADBM.class){
+      byte[] val = crsnaffle(name, key, key.length);
+      if(val == null) throw new CuriaException(crecode());
+      return val;
+    }
+  }
   //----------------------------------------------------------------
   // instance fields
   //----------------------------------------------------------------
@@ -657,6 +672,7 @@ public class Curia implements ADBM {
   private static synchronized final native int crvsizlob(int index, byte[] key, int ksiz);
   private static synchronized final native int crrnumlob(int index);
   private static synchronized final native int crremove(String name);
+  private static synchronized final native byte[] crsnaffle(String name, byte[] key, int ksiz);
 }
 
 
