@@ -133,6 +133,21 @@ public class Depot implements ADBM {
       if(dpremove(name) == 0) throw new DepotException(dpecode());
     }
   }
+  /**
+   * Retrieve a record directly from a database file.
+   * @param name the name of a database file.
+   * @param key a byte array of a key.
+   * @throws DepotException if an error occurs or no record corresponds.
+   * @note Although this method can be used even while the database file is locked by another
+   * process, it is not assured that recent updated is reflected.
+   */
+  public static byte[] snaffle(String name, byte[] key) throws DepotException {
+    synchronized(ADBM.class){
+      byte[] val = dpsnaffle(name, key, key.length);
+      if(val == null) throw new DepotException(dpecode());
+      return val;
+    }
+  }
   //----------------------------------------------------------------
   // instance fields
   //----------------------------------------------------------------
@@ -557,6 +572,7 @@ public class Depot implements ADBM {
   private static synchronized final native int dpinode(int index);
   private static synchronized final native long dpmtime(int index);
   private static synchronized final native int dpremove(String name);
+  private static synchronized final native byte[] dpsnaffle(String name, byte[] key, int ksiz);
 }
 
 
