@@ -153,12 +153,12 @@ void *_qdbm_mmap(void *start, size_t length, int prot, int flags, int fd, off_t
   static volatile long ready = FALSE;
   HANDLE handle;
   int i;
-  if(InterlockedExchange((long *)&first, FALSE)){
+  if(InterlockedExchange((void *)&first, FALSE)){
     InitializeCriticalSection(&mmcsec);
     atexit(_qdbm_delete_mmap_env);
-    InterlockedExchange((long *)&ready, TRUE);
+    InterlockedExchange((void *)&ready, TRUE);
   }
-  while(!InterlockedCompareExchange((long *)&ready, TRUE, TRUE)){
+  while(!InterlockedCompareExchange((void *)&ready, TRUE, TRUE)){
     Sleep(1);
   }
   if(fd < 0 || flags & MAP_FIXED) return MAP_FAILED;
@@ -230,7 +230,7 @@ static void _qdbm_delete_mmap_env(void){
 
 
 #elif defined(_SYS_FREEBSD_) || defined(_SYS_NETBSD_) || defined(_SYS_OPENBSD_) || \
-  defined(_SYS_RISCOS_) || defined(MYNOMMAP)
+  defined(_SYS_AIX_) || defined(_SYS_RISCOS_) || defined(MYNOMMAP)
 
 
 void *_qdbm_mmap(void *start, size_t length, int prot, int flags, int fd, off_t offset){
@@ -398,7 +398,7 @@ int _qdbm_win32_fcntl(int fd, int cmd, struct flock *lock){
   opt = (cmd == F_SETLK) ? LOCKFILE_FAIL_IMMEDIATELY : 0;
   if(lock->l_type == F_WRLCK) opt |= LOCKFILE_EXCLUSIVE_LOCK;
   memset(&ol, 0, sizeof(OVERLAPPED));
-  ol.Offset = 0;
+  ol.Offset = INT_MAX;
   ol.OffsetHigh = 0;
   ol.hEvent = 0;
   if(!LockFileEx(fh, opt, 0, 1, 0, &ol)){
