@@ -171,6 +171,24 @@ public:
    * @throw Curia_error if an error occurs.
    */
   static void remove(const char* name) throw(Curia_error);
+  /**
+   * Retrieve a record directly from a database directory.
+   * @param name the name of a database directory.
+   * @param kbuf the pointer to the region of a key.
+   * @param ksiz the size of the region of the key.  If it is negative, the size is assigned
+   * with `std::strlen(kbuf)'.
+   * @param sp the pointer to a variable to which the size of the region of the return value
+   * is assigned.  If it is 0, it is not used.
+   * @return the pointer to the region of the value of the corresponding record.
+   * @throw Curia_error if an error occurs or no record corresponds.
+   * @note Because an additional zero code is appended at the end of the region of the return
+   * value, the return value can be treated as a character string.  Because the region of the
+   * return value is allocated with the `std::malloc' call, it should be released with the
+   * `std::free' call if it is no longer in use.  Although this method can be used even while
+   * the database directory is locked by another process, it is not assured that recent updated
+   * is reflected.
+   */
+  static char* snaffle(const char* name, const char* kbuf, int ksiz, int* sp) throw(Curia_error);
   //----------------------------------------------------------------
   // public member functions
   //----------------------------------------------------------------
