@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Villa for Ruby
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -74,6 +74,8 @@ static VALUE rbvlcurnext(VALUE vself, VALUE vindex);
 static VALUE rbvlcurjump(VALUE vself, VALUE vindex, VALUE vkey, VALUE vjmode);
 static VALUE rbvlcurkey(VALUE vself, VALUE vindex);
 static VALUE rbvlcurval(VALUE vself, VALUE vindex);
+static VALUE rbvlcurput(VALUE vself, VALUE vindex, VALUE vval, VALUE vjmode);
+static VALUE rbvlcurout(VALUE vself, VALUE vindex);
 static VALUE rbvlsettuning(VALUE vself, VALUE vindex,
                            VALUE vlrecmax, VALUE vnidxmax, VALUE vlcnum, VALUE vncnum);
 static VALUE rbvlsync(VALUE vself, VALUE vindex);
@@ -147,6 +149,8 @@ Init_mod_villa(){
   rb_define_const(mvilla, "ONOLCK", INT2FIX(VL_ONOLCK));
   rb_define_const(mvilla, "OLCKNB", INT2FIX(VL_OLCKNB));
   rb_define_const(mvilla, "OZCOMP", INT2FIX(VL_OZCOMP));
+  rb_define_const(mvilla, "OYCOMP", INT2FIX(VL_OYCOMP));
+  rb_define_const(mvilla, "OXCOMP", INT2FIX(VL_OXCOMP));
   rb_define_const(mvilla, "CMPLEX", INT2FIX(0));
   rb_define_const(mvilla, "CMPDEC", INT2FIX(1));
   rb_define_const(mvilla, "CMPOBJ", INT2FIX(2));
@@ -154,8 +158,12 @@ Init_mod_villa(){
   rb_define_const(mvilla, "DKEEP", INT2FIX(VL_DKEEP));
   rb_define_const(mvilla, "DCAT", INT2FIX(VL_DCAT));
   rb_define_const(mvilla, "DDUP", INT2FIX(VL_DDUP));
+  rb_define_const(mvilla, "DDUPR", INT2FIX(VL_DDUPR));
   rb_define_const(mvilla, "JFORWARD", INT2FIX(VL_JFORWARD));
   rb_define_const(mvilla, "JBACKWARD", INT2FIX(VL_JBACKWARD));
+  rb_define_const(mvilla, "CPCURRENT", INT2FIX(VL_CPCURRENT));
+  rb_define_const(mvilla, "CPBEFORE", INT2FIX(VL_CPBEFORE));
+  rb_define_const(mvilla, "CPAFTER", INT2FIX(VL_CPAFTER));
   rb_define_module_function(mvilla, "mod_open", rbvlopen, 3);
   rb_define_module_function(mvilla, "mod_close", rbvlclose, 1);
   rb_define_module_function(mvilla, "mod_put", rbvlput, 4);
@@ -170,6 +178,8 @@ Init_mod_villa(){
   rb_define_module_function(mvilla, "mod_curjump", rbvlcurjump, 3);
   rb_define_module_function(mvilla, "mod_curkey", rbvlcurkey, 1);
   rb_define_module_function(mvilla, "mod_curval", rbvlcurval, 1);
+  rb_define_module_function(mvilla, "mod_curput", rbvlcurput, 3);
+  rb_define_module_function(mvilla, "mod_curout", rbvlcurout, 1);
   rb_define_module_function(mvilla, "mod_settuning", rbvlsettuning, 5);
   rb_define_module_function(mvilla, "mod_sync", rbvlsync, 1);
   rb_define_module_function(mvilla, "mod_optimize", rbvloptimize, 1);
@@ -488,6 +498,33 @@ static VALUE rbvlcurval(VALUE vself, VALUE vindex){
 }
 
 
+static VALUE rbvlcurput(VALUE vself, VALUE vindex, VALUE vval, VALUE vcpmode){
+  VILLA *villa;
+  const char *vbuf;
+  int index, vsiz, cpmode;
+  FIXNUM_P(vindex);
+  if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
+  vbuf = STR2CSTR(vval);
+  vsiz = RSTRING(vval)->len;
+  FIXNUM_P(vcpmode);
+  cpmode = FIX2INT(vcpmode);
+  villa = vltable[index];
+  if(!vlcurput(villa, vbuf, vsiz, cpmode)) myerror(dpecode);
+  return Qtrue;
+}
+
+
+static VALUE rbvlcurout(VALUE vself, VALUE vindex){
+  VILLA *villa;
+  int index;
+  FIXNUM_P(vindex);
+  if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
+  villa = vltable[index];
+  if(!vlcurout(villa)) myerror(dpecode);
+  return Qtrue;
+}
+
+
 static VALUE rbvlsettuning(VALUE vself, VALUE vindex,
                            VALUE vlrecmax, VALUE vnidxmax, VALUE vlcnum, VALUE vncnum){
   VILLA *villa;
